import re
from PIL import Image, ImageDraw

def parse_log(filename):
    faces = []
    with open(filename, 'r') as file:
        content = file.read()
        face_blocks = re.findall(r'Face \d+:(.*?)(?=Face \d+:|$)', content, re.DOTALL)
        
        for block in face_blocks:
            box = re.findall(r'Bounding box: \((\d+), (\d+)\) to \((\d+), (\d+)\)', block)[0]
            landmarks = re.findall(r'Point \d+: \((\d+), (\d+)\)', block)
            faces.append({
                'box': [int(x) for x in box],
                'landmarks': [(int(x), int(y)) for x, y in landmarks]
            })
    return faces

def draw_faces(image_path, faces):
    # 打开原始图像
    image = Image.open(image_path)
    draw = ImageDraw.Draw(image)

    for face in faces:
        # 绘制人脸边界框
        draw.rectangle([face['box'][0], face['box'][1], face['box'][2], face['box'][3]], outline="red", width=2)

        # 绘制面部特征点
        for point in face['landmarks']:
            draw.ellipse([point[0]-2, point[1]-2, point[0]+2, point[1]+2], fill="blue")

        # 添加标签
        labels = ["Left Eye", "Right Eye", "Nose", "Left Mouth", "Right Mouth"]
        for i, (point, label) in enumerate(zip(face['landmarks'], labels)):
            draw.text((point[0]+5, point[1]+5), label, fill="green")

    return image

# 原始图像路径
original_image_path = "input_image.jpg"  # 请确保这是你的原始图像文件名

# 解析日志文件
faces = parse_log('log')

# 在原图上绘制人脸
result_image = draw_faces(original_image_path, faces)

# 保存结果图像
result_image.save("face_detection_result.png")

print("图像已保存为 face_detection_result.png")